
#include "paircover.h"

PairCover::PairCover ( const unsigned int symbols )
	: m_symbols( symbols ), m_pairs( symbols * ( symbols - 1 ) / 2 )
{
	reset();
}

PairCover::~PairCover () {}


const PairCover & PairCover::operator= ( const PairCover & rhs )
{
	if( this == &rhs )
		return rhs;
	if( m_symbols != rhs.m_symbols )
		throw string( "paircover bad =" );

	m_pairset = rhs.m_pairset;
	return *this;
}


bool PairCover::cover ( const unsigned int sym1, const unsigned int sym2 )
{
	if( sym1 == sym2 || sym1 >= m_symbols || sym2 >= m_symbols )
		return false;
	const unsigned int pairi = pairToIndex( sym1, sym2 );
	if( pairi >= m_pairs )
		throw string( "paircover bad pair index" );
	return m_pairset.insert( pairi );
}


unsigned int PairCover::pairsTotal () const { return m_pairs; }

unsigned int PairCover::pairsCovered () const { return m_pairset.members(); }

bool PairCover::allCovered () const {	return m_pairset.members() == m_pairs; }

void PairCover::reset () { m_pairset.clear(); }


int PairCover::predictMerge ( const PairCover & addition ) const
{
	if( m_symbols != addition.m_symbols )
		throw( string( "paircover predictmerge mismatch" ) );
	return (int)addition.pairsCovered() - 2 * (int)m_pairset.collisions( addition.m_pairset );
}


unsigned int PairCover::pairToIndex ( const unsigned int sym1, const unsigned int sym2 )
{
	unsigned int symlow, symhigh;
	if( sym1 < sym2 )
	{
		symlow = sym1;
		symhigh = sym2;
	}
	else
	{
		symlow = sym2;
		symhigh = sym1;
	}
	const unsigned int offset = ( symlow * m_symbols ) + symhigh;
	return offset - ( ( symlow + 1 ) * ( symlow + 2 ) ) / 2;
}
