#ifndef PAIRCOVER_H_
#define PAIRCOVER_H_

#include "quickset.h"
#include <string>
using std::string;

class PairCover
{
	public:
		PairCover ( const unsigned int symbols );
			// create a set of all symbols-choose-2 pairs
			// consider all pairs not covered
		
		~PairCover ();
		
		const PairCover & operator= ( const PairCover & rhs );
		
		bool cover ( const unsigned int sym1, const unsigned int sym2 );
			// cover the specified pair, return true iff it was not already covered
			// also returns false for invalid (out of range, sym1=sym2) pairs
		
		unsigned int pairsTotal () const;
			// return number of pairs in the set
		
		unsigned int pairsCovered () const;
			// return number of pairs currently covered
		
		bool allCovered () const;
			// return true iff every pair is covered
		
		void reset ();
			// uncover all pairs
		
		int predictMerge ( const PairCover & addition ) const;
			// score how well adding addition's covers to this would work
			// +1 for every new cover, -1 for every re-cover
		
	private:
		
		unsigned int pairToIndex ( const unsigned int sym1, const unsigned int sym2 );
			// translate symbol pair to pairset index
		
		const unsigned int m_symbols;
		const unsigned int m_pairs;
		QuickSet m_pairset;
		
};

#endif /*PAIRCOVER_H_*/
