
#include "tsccd-ea.h"


void TSCCD_EA::breed ()
{
	TSCCD_Puzzle testpuz( m_params.tsccd_k, m_params.tsccd_v );
	TSCCD_SymMap map( m_params.tsccd_v );
	
	// generate random start block
	map.setRandom();
	testpuz.reset( map.mapBlock( TSCCD_Block( m_params.tsccd_k ) ) );
	
	///////////////////////////////////////////////
	// compile a full solution from individuals
	unsigned int run = 0; // run length with no errors
	std::vector< int > scorehistory;
	scorehistory.reserve( testpuz.solutionLength() );
	
	m_stats.constructions++;
	while( testpuz.movesLeft() > 0 )
	{
		// choose indv with most promise
		Individual * ind = getIndvForCover( testpuz );
		ind->setLastUse( m_stats.constructions );
		
		// get mapping
		map.captureMap( ind->getLeadin(), testpuz.getBlocks().back() );
		
		// apply remapped moves
		const std::vector< TSCCD_Move > & moves = ind->getMoves();
		int indscore = 0; // score for this individual's contribution
		unsigned int indmoves = 0; // moves actually used
		for( unsigned int i = 0; i < moves.size() && testpuz.movesLeft() > 0; i++ )
		{
			indmoves++;
			int score = testpuz.applyMove( map.mapMove( moves[i] ) );
			indscore += score;
			scorehistory.push_back( score );
			
			if( score == (int)testpuz.getK() - 1 )
				run++;
			else
			{
				if( run > m_stats.max_run )
					m_stats.max_run = run;
				run = 0;
			}
		}
		
		if( indmoves > 0 )
			ind->updateFitness( (float)indscore / (float)indmoves );
	}
	
	
	///////////////////////////////////////////////
	// check for new best solution
	if( testpuz.coveredPairs() > m_bestever->coveredPairs() )
	{
		*m_bestever = testpuz;
		if( m_bestever->isSolved() )
			m_stats.solved = true;
	}
	
	
	///////////////////////////////////////////////
	// create offspring
	const unsigned int movecount = testpuz.getMoves().size();
	for( unsigned int i = 0; i < m_params.offspring; i++ )
	{
		Individual newind( m_params.tsccd_k, m_params.tsccd_v );
		
		// pick a region to use as genome
		unsigned int starti, endi;
		randRange( starti, endi, movecount - 1, 2 );
		
		newind.setLeadin( testpuz.getBlocks()[ starti ] );
		
		int score = 0;
		for( unsigned int mi = starti; mi <= endi; mi++ )
		{
			newind.pushMove( testpuz.getMoves()[ mi ] );
			score += scorehistory[ mi ];
		}
		
		newind.setFitness( (float)score / (float)newind.getMoves().size() );
		newind.setLastUse( m_stats.constructions );
		
		m_pop.push_back( newind );
	}
}



/*
void TSCCD_EA::breed ()
{
	using namespace std;
	TSCCD_Puzzle testpuz( m_params.tsccd_k, m_params.tsccd_v );
	TSCCD_SymMap map( m_params.tsccd_v );
	
	std::vector< unsigned int > errorrecord;
	errorrecord.reserve( testpuz.solutionLength() );
	
	std::set< Individual * > usedup; // which indvs have been used already
	
	// generate random start block
	map.setRandom();
	testpuz.reset( map.mapBlock( TSCCD_Block( m_params.tsccd_k ) ) );
	
	
	///////////////////////////////////////////////
	// compile a full solution from individuals
	unsigned int run = 0; // run length with no errors
	while( testpuz.movesLeft() > 0 )
	{
		// choose indv with most similar start block
		Individual * ind = getIndvForBlock( testpuz.getBlocks().back(), usedup );
		usedup.insert( ind );
		
		// get mapping
		map.captureMap( ind->getLeadin(), testpuz.getBlocks().back() );
		
		// apply remapped moves
		const std::vector< TSCCD_Move > & moves = ind->getMoves();
		unsigned int inderrors = 0; // errors for this individual
		for( unsigned int i = 0; i < moves.size() && testpuz.movesLeft() > 0; i++ )
		{
			unsigned int errors = testpuz.applyMove( map.mapMove( moves[i] ) );
			errorrecord.push_back( errors );
			inderrors += errors;
			
			if( errors > 0 )
			{
				if( run > m_stats.max_run )
					m_stats.max_run = run;
				run = 0;
			}
			else
				run++;
		}
		
		// update this individual's fitness
		ind->setFitness( -inderrors, m_params.memoryrate );
	}
	m_stats.constructions++;
	
	
	///////////////////////////////////////////////
	// check for new best solution
	if( testpuz.coveredPairs() > m_bestever->coveredPairs() )
	{
		*m_bestever = testpuz;
		if( m_bestever->isSolved() )
			m_stats.solved = true;
	}
	
	
	///////////////////////////////////////////////
	// create offspring
	const unsigned int movecount = testpuz.getMoves().size();
	for( unsigned int i = 0; i < m_params.offspring; i++ )
	{
		Individual newind( m_params.tsccd_k, m_params.tsccd_v );
		
		// pick a region to use as genome
		unsigned int starti, endi;
		randRange( starti, endi, movecount - 1, 2 );
		
		newind.setLeadin( testpuz.getBlocks()[ starti ] );
		
		int errors = 0;
		for( unsigned int mi = starti; mi <= endi; mi++ )
		{
			newind.pushMove( testpuz.getMoves()[ mi ] );
			errors += errorrecord[ mi ];
		}
		
		newind.mutate( m_params.mutate );
		
		newind.setFitness( -errors, 0.0 );
		m_offspring.push_back( newind );
	}
}
*/
