
#include "tsccd-ea.h"

TSCCD_EA::Individual::Individual ( const unsigned int k, const unsigned int v )
	: m_k( k ), m_v( v ), m_leadin( k ),
		m_moves(), m_fitness( 0.0 ), m_lastuse( 0 )
{}


TSCCD_EA::Individual::Individual ( const TSCCD_EA::Individual & src )
	: m_k( src.m_k ), m_v( src.m_v ),
		m_leadin( src.m_leadin ), m_moves( src.m_moves ),
		m_fitness( src.m_fitness ), m_lastuse( src.m_lastuse )
{}


TSCCD_EA::Individual::~Individual ()
{}


const TSCCD_EA::Individual & TSCCD_EA::Individual::operator= ( const TSCCD_EA::Individual & rhs )
{
	if( this == &rhs )
		return rhs;
	if( m_k != rhs.m_k || m_v != rhs.m_v )
		throw string( "indv bad =" );
	m_leadin = rhs.m_leadin;
	m_moves = rhs.m_moves;
	m_fitness = rhs.m_fitness;
	m_lastuse = rhs.m_lastuse;
	return *this;
}


bool TSCCD_EA::Individual::operator< ( const Individual & rhs ) const
{
	if( getLastUse() == rhs.getLastUse() )
		return fitness() < rhs.fitness();
	else
		return getLastUse() < rhs.getLastUse();
}


void TSCCD_EA::Individual::setLeadin ( const TSCCD_Block & newleadin ) {	m_leadin = newleadin; }

const TSCCD_Block & TSCCD_EA::Individual::getLeadin () const { return m_leadin; }

const std::vector< TSCCD_Move > & TSCCD_EA::Individual::getMoves() const { return m_moves; }

void TSCCD_EA::Individual::pushMove ( const TSCCD_Move & newmove ) {	m_moves.push_back( newmove ); }


int TSCCD_EA::Individual::selfScore () const
{
	TSCCD_Puzzle testpuz( m_k, m_v );
	testpuz.reset( m_leadin );
	return testpuz.scoreMoves( m_moves, TSCCD_SymMap( m_v ) );
}


void TSCCD_EA::Individual::scramble ( const unsigned int length )
{
	m_moves.clear();
	m_moves.reserve( length );
	for( unsigned int i = 0; i < length; i++ )
		m_moves.push_back( TSCCD_Move( mt.randInt( m_k - 1 ), mt.randInt( m_v - 1 ) ) );
	
	TSCCD_SymMap map( m_v );
	map.setRandom();
	m_leadin = map.mapBlock( m_leadin );
}


bool TSCCD_EA::Individual::mutate ( const TSCCD_EA::Params::Mutate & params )
{
	bool change = false;
	
	if( mt.rand() < params.lenrate ) // add/del a gene?
	{
		if( m_moves.size() < 2 || mt.rand() < 0.5 )
			m_moves.push_back( TSCCD_Move( mt.randInt( m_k - 1 ), mt.randInt( m_v - 1 ) ) );
		else
			m_moves.pop_back();
		change = true;
	}
	else // mutate existing genes
		
		for( unsigned int i = 0; i < m_moves.size(); i++ )		
			if( mt.rand() < params.generate )
			{
				if( mt.rand() < params.posrate )
					m_moves[ i ] = TSCCD_Move( mt.randInt( m_k - 1 ), m_moves[ i ].symbol() );
				else
					m_moves[ i ] = TSCCD_Move( m_moves[ i ].pos(), mt.randInt( m_v  - 1 ) );
				change = true;
			}
	
	return change;
}


void TSCCD_EA::Individual::setFitness ( const float fitness )
{
	m_fitness = fitness;
	m_fitnessupdates = 1;
}


void TSCCD_EA::Individual::updateFitness ( const float fitness )
{
	m_fitnessupdates++;
	const float updaterate = 1.0 / (float)( m_fitnessupdates );
	m_fitness = updaterate * fitness + ( 1.0 - updaterate ) * m_fitness;
}


float TSCCD_EA::Individual::fitness () const { return m_fitness; }

void TSCCD_EA::Individual::setLastUse ( const unsigned int iteration ) { m_lastuse = iteration; }

unsigned int TSCCD_EA::Individual::getLastUse () const { return m_lastuse; }
