
#include "tsccd-ea.h"


TSCCD_EA::TSCCD_EA ()
{
	m_bestever = NULL;
}


TSCCD_EA::~TSCCD_EA ()
{
	if( m_bestever )
		delete m_bestever;
}


void TSCCD_EA::initialize ( const Params & params )
{
	m_params = params;
	
	// init pop
	m_pop.clear();
	Individual base( m_params.tsccd_k, m_params.tsccd_v );
	for( unsigned int i = 0; i < m_params.popsize; i++ )
	{
		base.scramble( mt.randInt( m_params.maxinitlen ) + 1 );
		base.setFitness( (float)base.selfScore() / (float)base.getMoves().size() );
		m_pop.push_back( base );
	}
	
	if( m_bestever )
		delete m_bestever;
	m_bestever = new TSCCD_Puzzle( m_params.tsccd_k, m_params.tsccd_v );
	
	m_stats.constructions = 0;
	m_stats.avg_fit = 0.0;
	m_stats.max_run = 0;
	m_stats.solved = false;
}


// void TSCCD_EA::breed ()  >>>  tsccd-ea-breed.cpp


void TSCCD_EA::select ()
{
	// duhhh
	/*while( m_offspring.size() > 0 )
	{
		Individual & existing = m_pop[ mt.randInt( m_pop.size() - 1 ) ];
		if( m_offspring.back().fitness() > existing.fitness() )
			existing = m_offspring.back();
		else
			if( mt.rand() < 0.2 )
				existing = m_offspring.back();
		m_offspring.pop_back();
	}*/
	
	std::sort( m_pop.begin(), m_pop.end() );
	m_pop.erase( m_pop.begin(), m_pop.begin() + ( m_pop.size() - m_params.popsize ) );
}


TSCCD_EA::Stats TSCCD_EA::getStats ()
{
	// calculate averages
	float totallen = 0.0;
	float totalfit = 0.0;
	std::vector< Individual >::iterator ind;
	for( ind = m_pop.begin(); ind != m_pop.end(); ind++ )
	{
		totallen += ind->getMoves().size();
		totalfit += ind->fitness();
	}
	m_stats.avg_len = totallen / (float)( m_pop.size() );
	m_stats.avg_fit = totalfit / (float)( m_pop.size() );
	
	m_stats.best_cover = (float)( m_bestever->coveredPairs() ) /
											 (float)( m_bestever->totalPairs() );
	
	return m_stats;
}


TSCCD_Puzzle * TSCCD_EA::getBest () { return m_bestever; }


TSCCD_EA::Individual * TSCCD_EA::getIndvForCover ( const TSCCD_Puzzle & puz )
{
	std::map< float, TSCCD_EA::Individual * > bests;
	TSCCD_SymMap mapping( puz.getV() );
	int bestscore = -(int)( puz.totalPairs() );
	
	std::vector< Individual >::iterator ind;
	for( ind = m_pop.begin(); ind != m_pop.end(); ind++ )
	{
		mapping.captureMap( ind->getLeadin(), puz.getBlocks().back() );
		int score = puz.scoreMoves( ind->getMoves(), mapping );
		if( score > bestscore )
		{
			bests.clear();
			bestscore = score;
		}
		if( score == bestscore )
			bests.insert( std::pair< float, TSCCD_EA::Individual * >( ind->fitness(), &(*ind) ) );
	}
	
	if( bests.size() == 0 )
		return NULL;
	return bests.rbegin()->second; // return most fit
}


TSCCD_EA::Individual * TSCCD_EA::getWorstIndv()
{
	/*Individual * worst = &( m_pop[0] );
	for( unsigned int i = 1; i < m_pop.size(); i++ )
		if( m_pop[i].fitness() < worst->fitness() )
			worst = &( m_pop[i] );
	return worst;*/
	return NULL;
}


void TSCCD_EA::randRange ( unsigned int & starti, unsigned int & endi,
								 					 const unsigned int maxi, const unsigned int minlen )
{
	do
	{
		starti = mt.randInt( maxi );
		endi = mt.randInt( maxi );
		if( starti > endi )
		{
			unsigned int x = starti;
			starti = endi;
			endi = x;
		}
	} while( endi - starti < minlen );
}
