
#include "tsccd-symmap.h"


TSCCD_SymMap::TSCCD_SymMap ( const unsigned int v )
	: m_v( v ), m_map( NULL )
{
	if( m_v > 0 )
	{
		m_map = new unsigned int[ m_v ];
		m_unmap = new unsigned int[ m_v ];
		setNormal();
	}
}


TSCCD_SymMap::~TSCCD_SymMap ()
{
	if( m_map )
		delete[] m_map;
	if( m_unmap )
		delete[] m_unmap;
}


void TSCCD_SymMap::captureMap ( const TSCCD_Block & fromblock,
																const TSCCD_Block & toblock )
{
	const unsigned int k = fromblock.getK();
	if( toblock.getK() != k )
		throw string( "symmap capture mismatch" );
	
	setNormal();
	for( unsigned int i = 0; i < k; i++ )
		setSingle( fromblock[ i ], toblock[ i ] );
}


void TSCCD_SymMap::setMap ( const std::vector< unsigned int > & tomap )
{
	if( tomap.size() != m_v )
		throw string( "symmap bad setmap" );
	
	for( unsigned int i = 0; i < m_v; i++ )
	{
		m_map[ i ] = tomap[ i ];
		m_unmap[ tomap[ i ] ] = i;
	}
}


void TSCCD_SymMap::setSingle ( const unsigned int fromsym, const unsigned int tosym )
{
	if( fromsym >= m_v || tosym >= m_v )
		throw string( "symmap bad setsingle" );
	
	unsigned int oldtosym = map( fromsym );
	if( oldtosym == tosym )
		return;
	
	m_map[ fromsym ] = tosym;
	m_map[ m_unmap[ tosym ] ] = oldtosym;
	
	m_unmap[ oldtosym ] = m_unmap[ tosym ];
	m_unmap[ tosym ] = fromsym;
}


void TSCCD_SymMap::setNormal ()
{
	for( unsigned int i = 0; i < m_v; i++ )
	{
		m_map[ i ] = i;
		m_unmap[ i ] = i;
	}
}


void TSCCD_SymMap::setRandom ()
{
	for( unsigned int i = 0; i < m_v; i++ )
		m_map[ i ] = i;

	for( unsigned int i = 0; i < m_v - 1; i++ )
	{
		unsigned int j = i + mt.randInt( m_v - 1 - i );
		if( j != i )
		{
			// swap i and j
			unsigned int x = m_map[ i ];
			m_map[ i ] = m_map[ j ];
			m_map[ j ] = x;
		}
	}
	
	// correct m_unmap
	for( unsigned int i = 0; i < m_v; i++ )
		m_unmap[ m_map[i] ] = i;
}


unsigned int TSCCD_SymMap::map ( const unsigned int sym ) const
{
	if( sym >= m_v )
		throw string( "symmap bad map" );
	
	return m_map[ sym ];
}


TSCCD_Block TSCCD_SymMap::mapBlock ( const TSCCD_Block & fromblock ) const
{
	std::vector< unsigned int > toblock;
	toblock.resize( fromblock.getK() );
	for( unsigned int i = 0; i < toblock.size(); i++ )
		toblock[ i ] = map( fromblock[ i ] );
	return TSCCD_Block( toblock );
}


TSCCD_Move TSCCD_SymMap::mapMove ( const TSCCD_Move & frommove ) const
{
	return TSCCD_Move( frommove.pos(), map( frommove.symbol() ) );
}


void TSCCD_SymMap::print ()
{
	using namespace std;
	for( unsigned int i = 0; i < m_v; i++ )
		cout << i << "  ";
	cout << endl;
	for( unsigned int i = 0; i < m_v; i++ )
		cout << m_map[i] << "  ";
	cout << endl;
	for( unsigned int i = 0; i < m_v; i++ )
		cout << m_unmap[i] << "  ";
	cout << endl;
}
