
#include "tsccd-block.h"


TSCCD_Block::TSCCD_Block ( const unsigned int k )
{
	m_k = k;
	m_syms = new unsigned int[ m_k ];
	for( unsigned int i = 0; i < m_k; i++ )
		m_syms[ i ] = i;
}


TSCCD_Block::TSCCD_Block ( const std::vector< unsigned int > symbols )
{
	m_k = symbols.size();
	m_syms = new unsigned int[ m_k ];
	for( unsigned int i = 0; i < m_k; i++ )
		m_syms[ i ] = symbols[ i ];
}


TSCCD_Block::TSCCD_Block ( const TSCCD_Block & src )
{
	m_k = src.m_k;
	m_syms = new unsigned int[ m_k ];
	for( unsigned int i = 0; i < m_k; i++ )
		m_syms[ i ] = src.m_syms[ i ];
}


TSCCD_Block::~TSCCD_Block ()
{
	if( m_syms )
		delete[] m_syms;
}


const TSCCD_Block & TSCCD_Block::operator= ( const TSCCD_Block & rhs )
{
	if( this == &rhs )
		return rhs;
	if( m_k != rhs.m_k )
	{
		if( m_syms )
			delete[] m_syms;
		m_k = rhs.m_k;
		m_syms = new unsigned int[ m_k ];
	}
	for( unsigned int i = 0; i < m_k; i++ )
		m_syms[ i ] = rhs.m_syms[ i ];
	return *this;
}


const unsigned int & TSCCD_Block::operator[] ( const unsigned int pos ) const
{
	if( pos >= m_k )
		throw string( "block bad index" );
	return m_syms[ pos ];
}


const unsigned int TSCCD_Block::getK () const { return m_k; }


void TSCCD_Block::applyMove ( const TSCCD_Move & move )
{
	if( move.pos() >= m_k )
		throw string( "block bad move pos" );
	m_syms[ move.pos() ] = move.symbol();
}


float TSCCD_Block::similarity ( const TSCCD_Block & other ) const
{
	if( m_k != other.getK() )
		throw string( "block size mismatch" );
	float match = 0.0;
	for( unsigned int i = 0; i < m_k; i++ )
		if( m_syms[i] == other[i] )
			match += 1.0;
	return match / (float)m_k;
}


void TSCCD_Block::print () const
{
	using namespace std;
	cout << "[ ";
	for( unsigned int i = 0; i < m_k; i++ )
		cout << m_syms[i] << " ";
	cout << "]";
}
